@@ -60,7 +60,7 @@ using namespace Qt::Literals::StringLiterals;
 enum Step {
     SPLASH,
     TERMS,
-    DISK,
+    INSTALLATION,
     REPLACE,
     PARTITIONS,
     ENCRYPTION,
@@ -138,12 +138,12 @@ MInstall::MInstall(MIni &acfg, const QCommandLineParser &args, const QString &cf
 MInstall::~MInstall() {
     if (oobe) delete oobe;
     if (base) delete base;
+    if (replacer) delete replacer;
     if (checkmd5) delete checkmd5;
     if (bootman) delete bootman;
     if (swapman) delete swapman;
     if (partman) delete partman;
     if (crypto) delete crypto;
-    if (replacer) delete replacer;
     if (autopart) delete autopart;
     if (throbber) delete throbber;
 }
@@ -208,8 +208,10 @@ void MInstall::startup()
         swapman = new SwapMan(proc, core, *partman, gui, this);
         autopart = new AutoPart(proc, core, partman, gui, appConf, this);
         partman->autopart = autopart;
-        replacer = new Replacer(proc, partman, gui, appConf);
-        connect(gui.radioEntireDisk, &QRadioButton::toggled, gui.boxAutoPart, &QGroupBox::setEnabled);
+        replacer = new Replacer(proc, partman, gui, *crypto, appConf);
+        //experimental tag
+        gui.radioReplace->setText(gui.radioReplace->text() + " (" + tr("Experimental","As In feature is not polished and may not work properly") + ")");
+        connect(gui.radioEntireDrive, &QRadioButton::toggled, gui.boxAutoPart, &QGroupBox::setEnabled);
         gui.labelConfirm->setText(tr("The %1 installer will now perform the requested actions.").arg(PROJECTNAME)
             + "<br/><img src=':/dialog-warning'/>"_L1 + tr("These actions cannot be undone. Do you want to continue?")
             + "<img src=':/dialog-warning'/>"_L1);
@@ -239,10 +241,10 @@ void MInstall::startup()
         // Build disk widgets
         partman->scan();
         autopart->scan();
-        replacer->scan();
-        if (gui.comboDisk->count() > 0) {
-            gui.comboDisk->setCurrentIndex(0);
-            gui.radioEntireDisk->setChecked(true);
+        replacer->scan(false);
+        if (gui.comboDriveSystem->count() > 0) {
+            gui.comboDriveSystem->setCurrentIndex(0);
+            gui.radioEntireDrive->setChecked(true);
             for (PartMan::Iterator it(*partman); *it; it.next()) {
                 if ((*it)->isVolume()) {
                     // found at least one partition
@@ -251,15 +253,12 @@ void MInstall::startup()
                 }
             }
         } else {
-            gui.radioEntireDisk->setEnabled(false);
+            gui.radioEntireDrive->setEnabled(false);
             gui.boxAutoPart->setEnabled(false);
             gui.radioCustomPart->setChecked(true);
         }
         // Override with whatever is in the config.
         loadConfig(1);
-        // Hibernation check box (regular install).
-        gui.checkHibernationReg->setChecked(gui.checkHibernation->isChecked());
-        connect(gui.checkHibernationReg, &QCheckBox::clicked, gui.checkHibernation, &QCheckBox::setChecked);
     }
     oobe->stashServices(true);
 
@@ -499,9 +498,9 @@ void MInstall::loadConfig(int stage) noexcept
 
     if (stage == 1) {
         // Automatic or Manual partitioning
-        config.setSection(u"Storage"_s, gui.pageDisk);
-        static constexpr const char *diskChoices[] = {"Drive", "Partitions"};
-        QRadioButton *const diskRadios[] = {gui.radioEntireDisk, gui.radioCustomPart};
+        config.setSection(u"Storage"_s, gui.pageInstallation);
+        static constexpr const char *diskChoices[] = {"Drives", "Partitions"};
+        QRadioButton *const diskRadios[] = {gui.radioEntireDrive, gui.radioCustomPart};
         config.manageRadios(u"Target"_s, 2, diskChoices, diskRadios);
 
         // Storage and partition management
@@ -536,13 +535,13 @@ void MInstall::saveConfig() noexcept
     config.setString(u"Product"_s, PROJECTNAME + ' ' + PROJECTVERSION);
 
     // Automatic or Manual partitioning
-    config.setSection(u"Storage"_s, gui.pageDisk);
-    static constexpr const char *diskChoices[] = {"Drive", "Partitions"};
-    QRadioButton *const diskRadios[] = {gui.radioEntireDisk, gui.radioCustomPart};
+    config.setSection(u"Storage"_s, gui.pageInstallation);
+    static constexpr const char *diskChoices[] = {"Drives", "Partitions"};
+    QRadioButton *const diskRadios[] = {gui.radioEntireDrive, gui.radioCustomPart};
     config.manageRadios(u"Target"_s, 2, diskChoices, diskRadios);
 
     // Storage and partition management
-    if(gui.radioEntireDisk->isChecked()) {
+    if(gui.radioEntireDrive->isChecked()) {
         autopart->manageConfig(config);
     } else {
         partman->saveConfig(config);
@@ -576,8 +575,8 @@ int MInstall::showPage(int curr, int next) noexcept
         gui.boxMain->unsetCursor();
     } else if (curr == Step::TERMS && next > curr) {
         if (modeOOBE) return Step::NETWORK;
-    } else if (curr == Step::DISK && next > curr) {
-        if (gui.radioEntireDisk->isChecked()) {
+    } else if (curr == Step::INSTALLATION && next > curr) {
+        if (gui.radioEntireDrive->isChecked()) {
             if (!autopart->validate(automatic, PROJECTNAME)) {
                 return curr;
             }
@@ -589,19 +588,30 @@ int MInstall::showPage(int curr, int next) noexcept
             return Step::PARTITIONS;
         }
     } else if (curr == Step::REPLACE && next > curr) {
-        if (!replacer->validate(automatic)) {
+        if (!replacer->validate()) {
             nextFocus = gui.tableExistInst;
             return curr;
         }
+        if (!replacer->preparePartMan()) {
+            return curr;
+        }
+        if (!pretend && !(base && base->saveHomeBasic())) {
+            QMessageBox msgbox(this);
+            msgbox.setIcon(QMessageBox::Critical);
+            msgbox.setText(tr("The data in /home cannot be preserved because"
+                " the required information could not be obtained."));
+            msgbox.exec();
+            return curr;
+        }
         return Step::CONFIRM;
+    } else if (curr == Step::REPLACE && next < curr) {
+        replacer->clean();
     } else if (curr == Step::PARTITIONS) {
         if (next > curr) {
-            gui.treeConfirm->clear();
             if (!partman->validate(automatic)) {
                 nextFocus = gui.treePartitions;
                 return curr;
             }
-            gui.treeConfirm->expandAll();
             if (!pretend && !(base && base->saveHomeBasic())) {
                 QMessageBox msgbox(this);
                 msgbox.setIcon(QMessageBox::Critical);
@@ -617,40 +627,31 @@ int MInstall::showPage(int curr, int next) noexcept
             }
             return Step::CONFIRM;
         }
-        return Step::DISK;
+        return Step::INSTALLATION;
     } else if (curr == Step::ENCRYPTION && next < curr) {
-        if (gui.radioEntireDisk->isChecked()) {
-            return Step::DISK;
+        if (gui.radioEntireDrive->isChecked()) {
+            return Step::INSTALLATION;
         }
         return Step::PARTITIONS;
     } else if (curr == Step::CONFIRM) {
+        gui.treeConfirm->clear(); // Revisiting a step produces a fresh confirmation list.
         if (next > curr) {
-            if (gui.radioEntireDisk->isChecked()) {
-                gui.checkHibernation->setChecked(gui.checkHibernationReg->isChecked());
-                partman->scan();
-                PartMan::Device *drive = autopart->selectedDrive();
-                if (!drive) {
+            if (gui.radioEntireDrive->isChecked()) {
+                if (!autopart->buildLayout()) {
                     gui.labelSplash->setText(tr("Cannot find selected drive."));
                     abortEndUI(false);
                     return Step::SPLASH;
                 }
-                autopart->buildLayout(drive, autopart->partSize(), gui.checkEncryptAuto->isChecked());
                 next = (advanced ? Step::BOOT : Step::SWAP);
-            } else if (gui.radioReplace->isChecked()) {
-                if (!replacer->preparePartMan()) {
-                    gui.labelSplash->setText(tr("Unable to prepare for replacement."));
-                    abortEndUI(false);
-                    return Step::SPLASH;
-                }
-            } else {
+            } else if (!gui.radioReplace->isChecked()) {
                 advanced = true;
             }
             bootman->buildBootLists();
             swapman->setupDefaults();
             loadConfig(2);
         } else {
-            if (gui.radioEntireDisk->isChecked()) {
-                return Step::DISK;
+            if (gui.radioEntireDrive->isChecked()) {
+                return Step::INSTALLATION;
             } else if (gui.radioCustomPart->isChecked()) {
                 return Step::PARTITIONS;
             } else if (gui.radioReplace->isChecked()) {
@@ -729,7 +730,7 @@ void MInstall::pageDisplayed(int next) noexcept
             + tr("Remember, this software is provided AS-IS with no warranty what-so-ever."
                 " It is solely your responsibility to backup your data before proceeding.") + "</p>"_L1);
         break;
-    case Step::DISK:
+    case Step::INSTALLATION:
         gui.textHelp->setText("<p><b>"_L1 + tr("Installation Options") + "</b><br/>"_L1
             + tr("If you are running Mac OS or Windows OS (from Vista onwards), you may have to use that system's software to set up partitions and boot manager before installing.") + "</p>"_L1
             "<p><b>"_L1 + tr("Using the root-home space slider") + "</b><br/>"_L1
@@ -747,7 +748,9 @@ void MInstall::pageDisplayed(int next) noexcept
             "<p><b>"_L1 + tr("Using a custom disk layout") + "</b><br/>"_L1
             + tr("If you need more control over where %1 is installed to, select \"<b>%2</b>\" and click <b>Next</b>."
                 " On the next page, you will then be able to select and configure the storage devices and"
-                " partitions you need.").arg(PROJECTNAME, gui.radioCustomPart->text().remove('&')) + "</p>"_L1);
+                " partitions you need.").arg(PROJECTNAME, gui.radioCustomPart->text().remove('&')) + "</p>"_L1
+                              + "<p>" + "<b>"_L1 + tr("Replace existing installation") + "</b><br/>"_L1 + tr("Replace existing installation option will attempt to replace an existing installation with the same "
+                                           "disk configuration as the existing installation.  Home directories are preserved."));
         break;
 
     case Step::REPLACE:
@@ -757,7 +760,7 @@ void MInstall::pageDisplayed(int next) noexcept
             + tr("There is no guarantee of this working successfully. Ensure you have a good working backup of all important data before continuing.") + "</p>"_L1);
 
         if(gui.tableExistInst->rowCount() <= 0) {
-            replacer->scan(true);
+            QTimer::singleShot(0, gui.pushReplaceScan, &QPushButton::click);
         }
         break;
 
@@ -860,6 +863,9 @@ void MInstall::pageDisplayed(int next) noexcept
     case Step::CONFIRM: // Confirmation and review.
         gui.textHelp->setText("<p><b>"_L1 + tr("Final Review and Confirmation") + "</b><br/>"_L1
             + tr("Please review this list carefully. This is the last opportunity to check, review and confirm the actions of the installation process before proceeding.") + "</p>"_L1);
+
+        gui.treeConfirm->expandAll();
+        gui.treeConfirm->resizeColumnToContents(0);
         if (!automatic) {
             core.sleep(500, true); // Prevent accidentally skipping the confirmation.
         }
@@ -870,14 +876,16 @@ void MInstall::pageDisplayed(int next) noexcept
             + tr("%1 uses the GRUB bootloader to boot %1 and Microsoft Windows.").arg(PROJECTNAME) + "</p>"_L1
             "<p>"_L1 + tr("By default GRUB is installed in the Master Boot Record (MBR) or ESP (EFI System Partition for 64-bit UEFI boot systems) of your boot drive and replaces the boot loader you were using before. This is normal.") + "</p>"_L1
             "<p>"_L1 + tr("If you choose to install GRUB to Partition Boot Record (PBR) instead, then GRUB will be installed at the beginning of the specified partition. This option is for experts only.") + "</p>"_L1
-            "<p>"_L1 + tr("If you uncheck the Install GRUB box, GRUB will not be installed at this time. This option is for experts only.") + "</p>"_L1);
+            "<p>"_L1 + tr("If you uncheck the Install GRUB box, GRUB will not be installed at this time. This option is for experts only.") + "</p>"_L1
+        "<p>"_L1 + tr("Generate host-specific initramfs will try to create an initramfs tailored for the particular device rather than a generic all-purpose initramfs. This option is for experts only.") + "</p>"_L1);
         enableBack = false;
         break;
 
     case Step::SWAP:
         gui.textHelp->setText("<p><b>"_L1 + tr("Create a swap file") + "</b><br/>"_L1
             + tr("A swap file is more flexible than a swap partition; it is considerably easier to resize a swap file to adapt to changes in system usage.") + "</p>"_L1
-            "<p>"_L1 + tr("By default, this is checked if no swap partitions have been set, and unchecked if swap partitions are set. This option should be left untouched, and is for experts only.") + "</p>"_L1);
+            "<p>"_L1 + tr("By default, this is checked if no swap partitions have been set, and unchecked if swap partitions are set. This option should be left untouched, and is for experts only.") + "</p>"_L1
+            "<p>"_L1 + tr("Zram swap is a method of putting swap space in RAM.  A compressed swap device is placed in RAM.  It may be used in conjunction with other forms of swap, or on its own.") + "</p>"_L1);
         enableBack = advanced;
         break;
 
